﻿-- Author: TeeKoo/Truck-King
-- Name: HatChanger.lua
-- Description: functions for player & playerStyle

-- Modifications only with my permission, Perkele!

-- Changelog:
-- 	v0.0.0.1 (20.1.2021):
--		- first Release
--
-- ToDo:
--

HatChanger = {};

function HatChanger.changeHat(self, actionName, inputValue)	
	local newHat = self.visualInformation.selectedHatIndex + inputValue
	local category	= self.selectedHatCategory
	
	if newHat < 1 then		-- Remove hat, only in category 0
		newHat = 0
	else
		newHat = math.max(newHat, category*200 + 1)
		newHat = math.min(newHat, (category+1) * 200)
	end
	self:setPlayerHatData(newHat)
end

function HatChanger.changeCategory(self, actionName, inputValue)
	if self.selectedHatCategory ~=nil then
		local newCategory = self.selectedHatCategory + inputValue
		newCategory = math.max(newCategory, 0)
		newCategory = math.min(newCategory, g_hatManager:getCategoryCount())
		if self.selectedHatCategory ~= newCategory then
			self:setPlayerHatData(newCategory*200 + 1)
		end
	end
end

function HatChanger:playerLoad(xmlFilename, playerStyle, creatorConnection, isOwner)
	if self.selectedHatCategory == nil then
		self.selectedHatCategory = 0;
	end
	
	if self.isServer then
		local uniqueUserId = g_currentMission.userManager:getUserByUserId(self.userId).uniqueUserId
		if g_hatManager:playerHasData(uniqueUserId) then
			self:setPlayerHatData(g_hatManager:getPlayerData(uniqueUserId, self.visualInformation.hatReferenceFilename), true)
		end
	end
end

function HatChanger:setPlayerHatData(hat, noEventSend)
	if hat ~= self.visualInformation.selectedHatIndex then
		if self.visualInformation:setHat(hat, self.isOwner) then
			PlayerSetHatEvent.sendEvent(self, hat, noEventSend)
			if self.isServer then
				local uniqueUserId = g_currentMission.userManager:getUserByUserId(self.userId).uniqueUserId
				g_hatManager:updatePlayerHat(uniqueUserId, self.selectedHatName, self.selectedHatCategory)
			end
		end
	end
end

function HatChanger:registerActionEvents()
    -- register action events for the player context without switching (important when this is called from within the UI context)
    g_inputBinding:beginActionEventsModification(Player.INPUT_CONTEXT_NAME)
	--								g_inputBinding:registerActionEvent(actionId, 								self, callback,			 		 triggerUp,	triggerDown,	triggerAlways,	startActive,	callbackState)
	_, self.changeHatEvent =		g_inputBinding:registerActionEvent(InputAction.AXIS_CHANGE_HAT,				self, HatChanger.changeHat, 	 false,		true,			false,			false,			nil)
	_, self.changeCategoryEvent =	g_inputBinding:registerActionEvent(InputAction.AXIS_CHANGE_HAT_CATEGORY,	self, HatChanger.changeCategory, false,		true,			false,			false,			nil)
	
    -- reset registration context, update event data in input system:
    g_inputBinding:endActionEventsModification()
end

function HatChanger:removeActionEvents()
	self.changeHatEvent = ""
	self.changeCategoryEvent = ""
end

function HatChanger:updateActionEvents()
	local allowHatChange = false
	if self.playerStateMachine:isAvailable("activateObject") then
		local activateObjectState = self.playerStateMachine:getState("activateObject")		
		if activateObjectState.object ~= nil then 
			if activateObjectState.object.sleepingTrigger ~= nil then 
				allowHatChange = true
			end
		end
	end

	if self.changeHatEvent ~= nil and self.changeCategoryEvent ~= nil then
		if allowHatChange then
			local hatName = self.selectedHatName
			local hatIndex = self.visualInformation.selectedHatIndex
			local categoryIndex = self.selectedHatCategory;
			local categoryName = g_hatManager:getCategoryRealName(categoryIndex)
			hatIndex = hatIndex - categoryIndex * 200
			
			if hatName == nil or hatName == "" then hatName = "UNKNOWN HAT" end
			if hatName == "no_hat" or hatIndex == 0 then hatName =  g_i18n:getText("no_hat") end
			if categoryName == nil or categoryName == "" then categoryName = "UNKNOWN CATEGORY" end
		
			g_inputBinding:setActionEventText(self.changeHatEvent, string.format("%s (%d: %s)", g_i18n:getText("input_AXIS_CHANGE_HAT"), hatIndex , hatName))
			g_inputBinding:setActionEventText(self.changeCategoryEvent, string.format("%s (%d: %s)", g_i18n:getText("input_AXIS_CHANGE_HAT_CATEGORY"), categoryIndex+1, categoryName))

			g_inputBinding:setActionEventActive(self.changeHatEvent, true)
			g_inputBinding:setActionEventActive(self.changeCategoryEvent, true)
		else
			g_inputBinding:setActionEventActive(self.changeHatEvent, false)	
			g_inputBinding:setActionEventActive(self.changeCategoryEvent, false)
		end
	end
end

function HatChanger:setHat(hatIndex, isOwn)
    local result = false
    if self.player ~= nil and self.playerHatNode ~= nil then
        if hatIndex == 0 and self.selectedHatIndex > 0 then
			if isOwn ~= nil then
			    if self.hatNode ~= nil then
					unlink(self.hatNode)
					delete(self.hatNode)
					self.hatNode = nil
					result = true
				end
			else
				result = self:removeHat()
			end
			if result then
				result = true
				self.player.selectedHatCategory = 0			
				self.selectedHatIndex = 0
				self.player.selectedHatName = "no_hat"
			end
			self:setHair(self.selectedHairIndex)

        elseif hatIndex ~= self.selectedHatIndex or self.hatNode == nil then
			local xmlFilename = ""
			local hatKey = ""
			local i3dFilename = ""
			local categoryIndex = math.floor(hatIndex/200)
			if categoryIndex > 0 then
				xmlFilename = Utils.getFilename(g_hatManager:getCategoryPath(categoryIndex))
				i3dFilename = g_hatManager:getCategoryFilePath(categoryIndex)
				hatKey = string.format("modHats.hat(%d)#", hatIndex - categoryIndex * 200 - 1)
			else
				xmlFilename = Utils.getFilename(self.hatReferenceFilename)
				hatKey = string.format("playerClothing.hats.hat(%d)#", hatIndex - 1)
			end
			local xmlFile = loadXMLFile("hatXML", xmlFilename)
			
			if isOwn == nil then
				self:removeHat()
			end
			if xmlFile ~= nil and xmlFile ~= 0 then
				if hasXMLProperty(xmlFile, hatKey .. "name") and hasXMLProperty(xmlFile, hatKey .. "filename") then
					i3dFilename = i3dFilename .. getXMLString(xmlFile, hatKey .. "filename")
					local hat = loadI3DFile(i3dFilename, false, false, false) -- Loaded first to local, to check file exist
					if hat ~= nil then
						if self.hatNode ~= nil then
							unlink(self.hatNode)
							delete(self.hatNode)
						end
						self.hatNode = hat
						setVisibility(self.hatNode, isOwn ~= true)
						
						self:setHair(0, true)
						link(self.playerHatNode, self.hatNode)
						self.selectedHatIndex = hatIndex
						self.player.selectedHatName = getXMLString(xmlFile, hatKey .. "name")
						self.player.selectedHatCategory = categoryIndex
						result = true
					end
				end
				delete(xmlFile)
			end
        end
    else
        g_logManager:devError("-- [PlayerStyle:setHat] Player not set. Missing call to PlayerStyle:loadXML().")
    end
    return result
end

function HatChanger.loadMission()
	if g_currentMission.missionInfo.savegameDirectory ~= nil then
		local savegameFilename = g_currentMission.missionInfo.savegameDirectory .. "/hats.xml"

		if fileExists(savegameFilename) then
			local xmlFile = loadXMLFile("playerHats", savegameFilename)
			local uniqueUserIdKey = ""
			local categoryNameKey = ""
			local hatNameKey = ""
			
			local uniqueUserId = ""
			local i = 0
			
			while (true) do
				uniqueUserIdKey = string.format("hats.player(%d)#uniqueUserId", i)
				categoryNameKey = string.format("hats.player(%d)#categoryName", i)
				hatNameKey = string.format("hats.player(%d)#hatName", i)
			
				if hasXMLProperty(xmlFile, uniqueUserIdKey) then
					uniqueUserId = getXMLString(xmlFile, uniqueUserIdKey)
					if uniqueUserId ~= nil then
						if hasXMLProperty(xmlFile, categoryNameKey) and hasXMLProperty(xmlFile, hatNameKey) then
							g_hatManager:registerPlayer(uniqueUserId, getXMLString(xmlFile, categoryNameKey), getXMLString(xmlFile, hatNameKey))
						end
					end
				else
					break
				end
				i = i + 1
			end
		end
	end
end 

function HatChanger.saveToXMLFile(missionInfo)
	local xmlFile = createXMLFile("hats", missionInfo.savegameDirectory .. "/hats.xml", "hats")
	
	if xmlFile ~= nil then
		g_hatManager:savePlayerHatData(xmlFile)
		saveXMLFile(xmlFile)
		delete(xmlFile)
	end
end 

function HatChanger:writeStream(streamId, connection)
    streamWriteUInt16(streamId, self.selectedHatIndex)
end

function HatChanger:readStream(streamId, connection)
    self.selectedHatIndex = streamReadUInt16(streamId)
end

PlayerStyle.readStream = Utils.appendedFunction(PlayerStyle.readStream, HatChanger.readStream);
PlayerStyle.writeStream = Utils.appendedFunction(PlayerStyle.writeStream, HatChanger.writeStream);
PlayerStyle.setHat = HatChanger.setHat;

Player.setPlayerHatData = HatChanger.setPlayerHatData
Player.load = Utils.appendedFunction(Player.load, HatChanger.playerLoad);
Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, HatChanger.registerActionEvents);
Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, HatChanger.removeActionEvents);
Player.updateActionEvents 	= Utils.appendedFunction(Player.updateActionEvents, HatChanger.updateActionEvents);

FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, HatChanger.saveToXMLFile)
Mission00.load = Utils.appendedFunction(Mission00.load, HatChanger.loadMission)


