﻿-- Author: TeeKoo/Truck-King
-- Name: HatManager.lua
-- Description: class for handling hats

-- Modifications only with my permission, Perkele!

-- Changelog:
-- 	v0.0.0.1 (20.1.2021):
--		- first Release
--
-- ToDo:
--

HatManager = {};
HatManager_mt = Class(HatManager)

function HatManager:new()
    local self = setmetatable({}, HatManager_mt)

	self.categorys = {}
	self.categorys[0] = {name = "original", realName = g_i18n:getText("original")}	
	self.players = {};

    return self
end

function HatManager:delete()
    self.categorys = nil;
	self.players = nil;
end

function HatManager:getCategoryCount()
	return table.getn(self.categorys)
end

function HatManager:getPlayerData(uniqueUserId, xmlPath)
	if self.players[uniqueUserId] ~= nil then
		local catName = self.players[uniqueUserId].category
		local index = nil
		for i,category in pairs (self.categorys) do
			if category.name == catName then
				index = i
				break
			end
		end;		
		
		local hatName = self.players[uniqueUserId].hat
		
		if index == 0 and hatName == "no_hat" then return 0 end		
		
		local xmlFilename = Utils.getFilename(xmlPath)
		local preFix = "playerClothing.hats"
		local key = ""
		
		if index ~= nil then
			if index > 0 then
				xmlFilename = Utils.getFilename(self:getCategoryPath(index))	
				preFix = "modHats"
			end
			
			if xmlFilename ~= nil and xmlFilename ~= "" then
				xmlFile = loadXMLFile("hatXML", xmlFilename)
		
				for i=1,200 do
					key = string.format("%s.hat(%d)#name", preFix, i - 1)
					if hasXMLProperty(xmlFile, key) then
						if getXMLString(xmlFile, key) == hatName then
							return index*200 + i;
						end
					end
				end
			end
		end
		g_logManager:error( string.format("player (%s) has unknown hat (category: %s, name: %s)", uniqueUserId, catName, hatName))
	end
	return 1
end

function HatManager:registerCategory(path, filePath, gln)
	local xmlFilename = Utils.getFilename(filePath .. path)

	if xmlFilename ~= nil then
		if fileExists(xmlFilename) then
			local xmlFile = loadXMLFile("hatXML", xmlFilename)
			local nameKey = "modHats#categoryName"
			
			if hasXMLProperty(xmlFile, nameKey) then
				local name = getXMLString(xmlFile, nameKey)
				if name ~= nil and name ~= "" then
					local newCategory = {};
					newCategory.name = name;
					newCategory.path = filePath .. path;
					newCategory.filePath = filePath;
					if gln ~= nil then
						newCategory.realName = gln:getText(name)
					end;
					table.insert(self.categorys, newCategory)
					print(string.format("New hat category '%s' loaded",name))
				end
			end
		end
	end
end

function HatManager:getCategoryName(index)
	if self.categorys[index] ~= nil then
		return self.categorys[index].name
	end
	return "NULL"
end

function HatManager:getCategoryRealName(index)
	if self.categorys[index] ~= nil then
		local name = self.categorys[index].realName
		if name ~= nil and name ~= "" then return name end				
		return self.categorys[index].name
	end
	return ""
end

function HatManager:getCategoryPath(index)
	if self.categorys[index] ~= nil then
		return self.categorys[index].path
	end
	return ""
end

function HatManager:getCategoryFilePath(index)
	if self.categorys[index] ~= nil then
		return self.categorys[index].filePath
	end
	return ""
end

function HatManager:playerHasData(uniqueUserId)
	return self.players[uniqueUserId] ~= nil
end

function HatManager:registerPlayer(id, categoryName, hatName)
	if self.players[id] == nil then
		self.players[id] = {}
		self.players[id].category = categoryName
		self.players[id].hat = hatName
	end;
end

function HatManager:updatePlayerHat(uniqueUserId, hatName, category)
	if uniqueUserId ~= "" and uniqueUserId ~= nil then
		if self.players[uniqueUserId] == nil then
			self:registerPlayer(uniqueUserId, "", "")
		end
	
		self.players[uniqueUserId].category = self:getCategoryName(category)
		self.players[uniqueUserId].hat = hatName	
	end
end

function HatManager:savePlayerHatData(xmlFile)
	local count = 0
	for uniqueUserId,data in pairs (self.players) do
		if data ~= nil then
			setXMLString(xmlFile, string.format("hats.player(%d)#uniqueUserId", count), uniqueUserId)		
			setXMLString(xmlFile, string.format("hats.player(%d)#categoryName", count), data.category)
			setXMLString(xmlFile, string.format("hats.player(%d)#hatName", count), data.hat)
			count = count + 1
		end
	end;
end




